/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is bookmarktools.
 *
 * The Initial Developer of the Original Code is
 * Maik Gollnick <maik.gollnick@gmx.de>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
 
var Encoding = {

	/*
		Returns true if the given mimetype is supported, otherwise false.
		@pre typeof mimeType != 'undefined' && mimeType != null && mimeType.toLowerCase()
	*/
	supportedMimeType : function(mimeType){
		switch(mimeType){
			case "application/rtf" : return true;
			case "application/xhtml+xml" : return true;
			case "application/xml" : return true;
			case "application/x-javascript" : return true;
			case "application/rss+xml" : return true;
			case "text/comma-separated-values" : return true;
			case "text/css" : return true;
			case "text/html" : return true;
			case "text/javascript" : return true;
			case "text/plain" : return true;
			case "text/rtf" : return true;
			case "text/tab-separated-values" : return true;
			case "text/xml" : return true;
			default : return false;
		}
	},

	/*
		Returns true if the given charset is supported, otherwise false.
		@pre typeof charSet != 'undefined' && charSet != null && charSet.toLowerCase()
	*/
	supportedCharSet : function(charSet){
		switch(charSet){
			case "charset=us-ascii" : return true;
			case "charset=utf-8" : return true;
			case "charset=iso-8859-1" : return true;
			case "charset=iso-8859-2" : return true;
			case "charset=iso-8859-3" : return true;
			case "charset=iso-8859-4" : return true;
			default : return false;
		}
	},
	
	isRTF : function(mimeType){
		return mimeType == "application/rtf" || mimeType == "text/rtf";
	},
	
	isHTML : function(mimeType){
		return mimeType == "text/html";
	},
	
	isXML : function(mimeType){
		return mimeType == "application/xhtml+xml" || 
			mimeType == "application/xml" ||
			mimeType == "application/rss+xml" ||
			mimeType == "text/xml";
	},
	
	isPLAIN : function(mimeType){
		return mimeType == "application/x-javascript" || 
			mimeType == "text/comma-separated-values" ||
			mimeType == "text/css" ||
			mimeType == "text/javascript" ||
			mimeType == "text/plain" ||
			mimeType == "text/tab-separated-values";
	},
	
	/*
		Charset independend entity filter.
	*/
	maskedEntity : function(entity){
		switch(entity){
			case "&Agrave;" : // 
			case "&Aacute;" : // 
			case "&Acirc;" : // 
			case "&Atilde;" : // 
			case "&Auml;" : // 
			case "&Aring;" : // 
				return 'A';
			case "&AElig;" : // 
				return "AE";
			case "&Ccedil;" : // 
				return 'C';
			case "&Egrave;" : // 
			case "&Eacute;" : // 
			case "&Ecirc;" : // 
			case "&Euml;" : // 
				return 'E';
			case "&Igrave;" : // 
			case "&Iacute;" : // 
			case "&Icirc;" : // 
			case "&Iuml;" : // 
				return 'I';
			case "&ETH;" : // 
				return 'D';
			case "&Ntilde;" : // 
				return 'N';
			case "&Ograve;" : // 
			case "&Oacute;" : // 
			case "&Ocirc;" : // 
			case "&Otilde;" : // 
			case "&Ouml;" : // 
			case "&Oslash;" : // 
				return 'O';
			case "&THORN;" : // 
				return "TH";
			case "&Ugrave;" : // 
			case "&Uacute;" : // 
			case "&Ucirc;" : // 
			case "&Uuml;" : // 
				return 'U';
			case "&Yacute;" : // 
				return 'Y';
			case "&agrave;" : // 
			case "&aacute;" : // 
			case "&acirc;" : // 
			case "&atilde;" : // 
			case "&auml;" : // 
			case "&aring;" : // 
				return 'a';
			case "&aelig;" : // 
				return "ae";
			case "&ccedil;" : // 
				return 'c';
			case "&egrave;" : // 
			case "&eacute;" : // 
			case "&ecirc;" : // 
			case "&euml;" : // 
				return 'e';
			case "&igrave;" : // 
			case "&iacute;" : // 
			case "&icirc;" : // 
			case "&iuml;" : // 
				return 'i';
			case "&eth;" : // 
				return 'd';
			case "&ntilde;" : // 
				return 'n';
			case "&ograve;" : // 
			case "&oacute;" : // 
			case "&ocirc;" : // 
			case "&otilde;" : // 
			case "&ouml;" : // 
			case "&oslash;" : // 
				return 'o';
			case "&szlig;" : // 
				return "ss";
			case "&thorn;" : // 
				return "th";
			case "&ugrave;" : // 
			case "&uacute;" : // 
			case "&ucirc;" : // 
			case "&uuml;" : // 
				return 'u';
			case "&yacute;" : // 
			case "&yuml;" : // 
				return 'y';
			default : break;
		}
		return null;
	},
	
	/*
		Filter for standard entities.
	*/
	standardEntity : function(entity){
		switch(entity){
			case "&quot;" : return '"';
			case "&#34;" : return '"';
			case "&amp;" : return '&';
			case "&#38;" : return '&';
			case "&lt;" : return '<';
			case "&#60;" : return '<';
			case "&gt;" : return '>';
			case "&#62;" : return '>';
			default : break;
		}
		return null;
	},
	
	/*
		Extracts all document entities as array.
	*/
	extractEntities : function(markup){
		var match = markup.match(/&([a-zA-Z0-9#]+);/gi);
		return match ? match : null;
	},
	
	/*
		Filters all known entities with ascii characters, otherwise the unknown
		entity is replaced by an empty string.
	*/
	filterEntities : function(responseText){
		var entities = Encoding.extractEntities(responseText);
		if(entities != null){
			for(var i = 0, l = entities.length ;i < l; i++){
				var entity = entities[i];
				var standardEntity = Encoding.standardEntity(entity);
				var maskedEntity = Encoding.maskedEntity(entity);
				
				if(standardEntity != null || maskedEntity != null){
					var replaceEntity = maskedEntity != null ? maskedEntity : standardEntity;
					responseText = responseText.replace(entity, replaceEntity);
				}else {
					responseText = responseText.replace(entity, StringUtils.EMPTY);
				}
			}
		}
		return responseText;
	}
	
};

var Parser = {

	tidy : null,

	init : function(){
		var instTidy = Cc["@bookmarktools.mozdev.org/repair/tidy;1"].createInstance();
		Parser.tidy = instTidy.QueryInterface(Ci.ITidy);
	},
	
	_init : function(){
		Parser.tidy = null;
	},

	removeWhiteSpaces : function(content){
		return content.replace(/(\s)+/gi, StringUtils.EMPTY);
	},
	
	removeNumbers : function(content){
		return content.replace(/[0-9]+/gi, StringUtils.EMPTY);
	},
	
	replaceWhiteSpaces : function(content){
		return content.replace(/(\s)+/gi, ' ');
	},
	
	parseRTF : function(responseText){
		responseText = responseText.replace(/\{?\\[a-zA-Z0-9*-]+\s*/gi, StringUtils.EMPTY);
		responseText = responseText.replace(/\s*[a-zA-Z\s]+;\}/gi, StringUtils.EMPTY);
		return responseText.replace(/\}/gi, StringUtils.EMPTY);
	},

	parse : function(mimeType, charSet, responseText){
		if(Encoding.isPLAIN(mimeType)){
			return responseText;
		}else {
			if(Encoding.isRTF(mimeType)){
				return Parser.parseRTF(responseText);
			}else {
				if(Encoding.isHTML(mimeType) || Encoding.isXML(mimeType)){
					var response = Parser.tidy.cleanup(responseText);
					if(response != "Tidy: no output"){
						response = Parser.replaceWhiteSpaces(response);
						response = StringUtils.stripScripts(response); 
						response = StringUtils.stripTags(response); 
						response = Encoding.filterEntities(response);
						return response;
					}else {
						return responseText;
					}
				}else {
					throw "Error(Parser): Unsupported mimetype: " + mimeType;
				}
			}
		}
	}
	
}; 
